import os
import numpy as np
import iris
import iris.cube
import iris.analysis
import warnings
import numba
import datetime
from numba import jit, njit
from numba import vectorize
from numba import guvectorize
from numba import cuda
from numba import int32, int64, float32, float64
from numba.cuda.cudadrv import driver

MESH_FILE = "/esarchive/autosubmit/con_files/mesh_mask_nemo.Ec3.2_O1L75.nc"
DATA_FILE = "/esarchive/exp/ecearth/a13c/cmorfiles/CMIP/EC-Earth-Consortium" \
            "/EC-Earth3-LR/historical/r1i1p1f1/Omon/tos/gn/v20180501" \
            "/tos_Omon_EC-Earth3-LR_historical_r1i1p1f1_gn_195001-195012.nc"
REGIONS_FILE = "/esarchive/autosubmit/con_files/mask.regions.Ec3.2_O1L75.nc"


def main():
    start = datetime.datetime.now()
    e1t, e2t, mask = load_mesh()
    weights = compute_weights(mask, e1t, e2t)
    del mask, e1t, e2t
    tos = load_tos()
    weighted_tos, vector_weight = variable_weight(tos, weights)
    reduced_weight = sum_red.reduce(vector_weight, axis=0)
    reduced_weighted_tos = sum_red.reduce(weighted_tos, axis=1)
    regmean = compute_regmean_cpu(reduced_weighted_tos, reduced_weight)
    end = datetime.datetime.now() - start
    print(regmean, end)


def load_mesh():
    e1t = iris.load_cube(MESH_FILE, 'e1t').data.astype(np.float32)
    e2t = iris.load_cube(MESH_FILE, 'e2t').data.astype(np.float32)
    mask = iris.load_cube(MESH_FILE, 'tmask')[0, 0, :].data.astype(np.float32)
    return e1t, e2t, mask


def load_masks():
    basins = {}
    cubes = iris.load(REGIONS_FILE)
    for cube in cubes:
        name = cube.name()
        if name in ('nav_lat', 'nav_lon'):
            continue
        cube = cube.extract(iris.Constraint(z=1))
        basins[name] = cube.data.astype(np.float32)
    return basins


def load_tos():
    tos = iris.load_cube(DATA_FILE, 'sea_surface_temperature')
    tos_data = tos.data.astype(np.float32)
    return tos_data


@vectorize(['float32(float32, float32, float32)'], target='cpu')
def compute_weights(mask, e1t, e2t):
    weights = mask*e1t*e2t
    return weights


@njit
def variable_weight(var, weight):
    times = var.shape[0]
    lats = var.shape[1]
    lons = var.shape[2]
    weighted_var = np.empty((times, lats*lons))
    vector_weight = np.empty(lats*lons)
    for t in range(times):
        for j in range(lats):
            for i in range(lons):
                x = i+j*var.shape[2]
                value = var[t, j, i] * weight[0, j, i]
                weight_value = weight[0, j, i]
                weighted_var[t, x] = value
                vector_weight[x] = weight_value
    return weighted_var, vector_weight


@vectorize(['float64(float32, float32)', 'float64(float64, float64)'],
           target='cpu')
def sum_red(x, y):
    return x+y


@vectorize(['float32(float64, float64)'], target='cpu')
def compute_regmean_cpu(weighted_var, weights):
    regmean = weighted_var/weights
    return regmean


if __name__ == '__main__':
    main()
